# -*- coding: latin-1 -*-

from sys import platform
import re
import os

from .lyrics_tokenizer import tokenize

def midi2xml(lyrics, midi_path, xml_path, tempo):
	temp_xml = "temp.xml"
	create_music_xml(midi_path, temp_xml)
	lyrics = tokenize(lyrics, midi_path)
	generate_voice_specification(lyrics, tempo, temp_xml, xml_path)
	os.remove(temp_xml)

def create_music_xml(midi_path, new_musicxml_path):
	if platform in ["win32","cygwin"]:
		os.system("MuseScore3.exe "+ midi_path +" -o " + new_musicxml_path)
	elif platform == "darwin":
		os.system("export QT_QPA_PLATFORM=offscreen && mscore "+ midi_path +" -o " + new_musicxml_path)
	else:
		os.system("export QT_QPA_PLATFORM=offscreen && musescore "+ midi_path +" -o " + new_musicxml_path)

def generate_voice_specification(lyrics, tempo, input_music_xml_path, output_music_xml_path):
	with open(input_music_xml_path, 'r') as c:
		content = [x.strip() for x in c.readlines()]
		with open(output_music_xml_path,'w') as f:
			f.write(add_voice_tags(tempo, lyrics, content))

def add_voice_tags(tempo, text, content):
	output = ""
	lyrics_xml = '<voice>1</voice>\n<lyric>\n<syllabic>{}</syllabic>\n<text>{}</text>\n</lyric>\n'

	i = 0
	ignore_this_note = False
	prev_beginning = False
	for line in content:
		if "<rest/>" in line or '<tie type="stop"/>' in line:
			ignore_this_note = True
		if "</note" in line:
			if not ignore_this_note:
				next_part = text[i%len(text)]
				if next_part.endswith("-"):
					next_part = next_part[:-1]
					if prev_beginning:
						syllabic = "middle"
					else:
						syllabic = "begin"
						prev_beginning = True
				else:
					if prev_beginning:
						syllabic = "end"
					else:
						syllabic = "single"
					prev_beginning = False
				output += lyrics_xml.format(syllabic, next_part)
				i+=1
			else:
				ignore_this_note = False

		output += line

	output = re.sub(r"<per-minute>\d+</per-minute>", f"<per-minute>{tempo}</per-minute>",output)
	output = re.sub(r'<sound tempo="\d+"/>', f'<sound tempo="{tempo}"/>',output)

	return output



import argparse

from . import renderize_voice

ap = argparse.ArgumentParser()

ap.add_argument("-l", "--lyrics", required=True,
   help="Path to txt file containing the lyrics")
ap.add_argument("-m", "--midi", required=True,
   help="Path to midi file")
ap.add_argument("-g", "--gender", required=False, default="female", choices=['female', 'male'],
   help="Gender voice (female/male)")
ap.add_argument("-t", "--tempo", required=False, default=80,
   help="Song tempo in BPMs")
ap.add_argument("-d", "--destination_folder", required=False, default='.',
   help="Destination folder")
args = vars(ap.parse_args())

with open(args['lyrics'], 'r') as text:
	lyrics = text.readlines()

renderize_voice(lyrics, args['midi'], args['gender'], int(args['tempo']), args['destination_folder'])


from collections import defaultdict

from music21 import converter, instrument, note, chord
import pyphen

dic = pyphen.Pyphen(lang='en')

def tokenize(text,midiPath):
	new_text = "" 
	i = 0
	for n in notesPerVerse(midiPath):
		verse = cleanText(text[i])
		new_text += "|".join(vocals(verse,n)) + "|"
		i = (i+1)%len(text)

	new_text = new_text.strip()
	return list(new_text.split("|"))

def notesPerVerse(midiFile):

	mid = converter.parse(midiFile)

	instruments = instrument.partitionByInstrument(mid)

	assert len(instruments.parts) == 1 # MIDI file must contain only vocals

	for instrument_i in instruments.parts:
	    notes_to_parse = instrument_i.recurse()

	notes_per_verse = 8

	notes_to_parse = list(filter(lambda element : isinstance(element, note.Note) or isinstance(element, chord.Chord), notes_to_parse))

	firstBar = int(notes_to_parse[0].offset/4)

	notesPerCompass = defaultdict(int)
	for element in notes_to_parse:
		start = element.offset
		notesPerCompass[int((start-4*firstBar)/notes_per_verse)] += 1

	return list(notesPerCompass.values())

continuation = "-"

def extendWord(text):
	for extensibleEnding in ["all","oh","ah","ad","as","at","al","a","e","i","o","u"]:
		if text.endswith(extensibleEnding) or text.endswith(extensibleEnding+continuation):
			if text.endswith(continuation):
				additional = continuation
			else:
				additional = ""

			if len(extensibleEnding) > 1:
				return text[:-1*len(extensibleEnding) + 1 - len(additional)] + continuation, extensibleEnding + additional
			else:
				return text, extensibleEnding
	return text, None

def findSmallers(silabas):
    smallers = -1
    min = float('inf')
    for i in range(len(silabas)-1):
        if min > len(silabas[i]) + len(silabas[i+1]):
            min = len(silabas[i]) + len(silabas[i+1])
            smallers = i
    return i

def silabas_word(text):
    return dic.inserted(text).replace("-", continuation + "|").split("|")

def silabas_sentence(sentence):
    return sum([silabas_word(word) for word in sentence.split()],[])

def vocals(text,n):
	silabas = silabas_sentence(text)

	if len(silabas) == n:
	    return silabas
	elif len(silabas) < n:
	    if all([extendWord(silaba)[1] == None for silaba in silabas]):
	    	silabas += ["a"]	    	
	    index = len(silabas)-1
	    while(len(silabas) < n):
	        prevPart, extensiblePart = extendWord(silabas[index])
	        if extensiblePart:
	        	silabas = silabas[:index] + [prevPart, extensiblePart] + silabas[index+1:]
	        index = (index - 1) % len(silabas)         
	    return silabas
	else:
	    while(len(silabas) > n):
	        i = findSmallers(silabas)
	        silabas = silabas[:i] + [silabas[i] + silabas[i+1]] + silabas[i+2:]
	    return silabas     

def cleanText(text):
	text = text.lower()

	symbolsToDelete = '.,!?"'
	for symbol in symbolsToDelete:
		text = text.replace(symbol,"")

	return text



import os
import sys
import urllib.request
import requests

from .midi2xml import midi2xml

def renderize_voice(lyrics, midi_path, sex="female", tempo=80, out_folder="."):
	VOICE_XML_PATH = os.path.join(out_folder,"voice.xml")
	VOICE_WAV_PATH = os.path.join(out_folder,"voice.wav")

	midi2xml(lyrics,midi_path,VOICE_XML_PATH,tempo)
	sinsy_request(VOICE_XML_PATH, VOICE_WAV_PATH,sex)

def sinsy_request(xml_file_path, wav_path, sex):
	if sex == "male":
		SPKR = 5
	else:
		SPKR = 4

	headers = {'User-Agent': 'Mozilla/5.0'}
	payload = {'SPKR_LANG':'english', 'SPKR':SPKR, 'VIBPOWER':'1', 'F0SHIFT':'0'}
	files = {'SYNSRC': open(xml_file_path,'rb')}

	# Sending post request and saving response as response object 
	r = requests.post(url='http://sinsy.sp.nitech.ac.jp/index.php', headers=headers, data=payload, files=files)
	html_response = r.text.split("temp/")

	# Magic scraping of the website to find the name of the wav file generated
	url_file_name = find_wav_name_on_website(html_response)

	if url_file_name is None:
		raise Exception("No wav file found on sinsy.jp")
	else:
		download(url_file_name, wav_path)

def find_wav_name_on_website(htmlResponse):
	url_file_name = None
	for line in htmlResponse:
		parts = line.split(".")
		if parts[1][:3] == "wav":
			url_file_name = parts[0]
			break
	return url_file_name

def download(url_file_name, wav_path):
	urllib.request.urlretrieve("http://sinsy.sp.nitech.ac.jp/temp/" + url_file_name + ".wav", wav_path)